﻿using UnityEngine;
using System.Collections;
using System.Linq; 
using U2DProjectCore;

public class NoteScript : MonoBehaviour {

	// Variables
	private Vector2 lastTarget;
	private Vector2 nextTarget;
	private int nodeIndex; 
	private float timeInterval = .05f; 
	private float currentInterval = 0;
	private bool colliderLock; 

	// Reference Variables
	private Spawnable_Note spawnScript;
	
	
	// [[ ----- START ----- ]]
	private void Start(){

		// init refs
		spawnScript = GetComponent<Spawnable_Note>(); 

		InitNote();

	}

	// [[ ----- INIT NOTE ----- ]]
	public void InitNote(){
		lastTarget = WaveController.nodeList[nodeIndex].transform.position;
		nextTarget = WaveController.nodeList[nodeIndex+1].transform.position; 
		nodeIndex = 0; 
	}


	// [[ ----- UPDATE ----- ]]
	private void Update(){
		currentInterval += Time.deltaTime;

		// move
		transform.position = Vector2.Lerp( 
			lastTarget, nextTarget, currentInterval /** Time.deltaTime*/);

		// update target at interval end
		if( currentInterval >= timeInterval ){
			currentInterval = 0;

			nodeIndex++;
			// if made it to the end, restart the cycle
			if( nodeIndex >= WaveController.nodeList.Count - 1){
				ResetNote();
			}

			lastTarget = WaveController.nodeList[nodeIndex].transform.position;
			nextTarget = WaveController.nodeList[nodeIndex+1].transform.position; 
		}

	}

	// [[ ----- RESET NOTE ----- ]]
	private void ResetNote(){
		spawnScript.SetPooled(); // print("Pooled: " + gameObject.name +" NodeIndex: " + nodeIndex); 
		nodeIndex = 0;
	}

	// [[ ----- ON TRIGGER ENTER 2D ----- ]]
	private void OnTriggerEnter2D( Collider2D _c ){
		if( _c.gameObject.tag == "obstacle" ){ ResetNote(); }
		if( _c.gameObject.tag == "enemy" ){ ResetNote(); }
	}


}
