﻿using UnityEngine;
using System.Collections.Generic;
using UnityEngine.UI;

namespace U2DProjectCore { 
	/**
	@brief Used to view contents of SQL Database and test read/write functionality 

	Part of Project Core Unity package. <br>
	This is a version of a test script designed for testing the Score Database. <br>
	Sends methods to the DataService class that manipulates entries in an SQL 
	database.  Use the methods here to test query functionality of DataService 
	methods. Use ds.CreateScoreDB() or similar method to create or re-initialize a 
	database - this should be done before creating a shippable build of the game so
	that the database does not contain test data. 

	_Component of:_
	<br> Main Camera

	_Class output:_
	<br> Calls methods of DataService which handles SQLite logic

	_Class input:_
	<br> none

	@author Jonny Aiello
	@version 0.1.1
	@date 5/1/16
	*/ 
	public class ScoreDatabaseManager : MonoBehaviour {

		public Text DebugText;

		// [[ ----- START ----- ]]
		void Start () {
			
			// instantiate DataService - string = name of database you connect to
			var ds = new DataService("scoreData.db");

			// create new database - use to re-initialize blank DB before building
			// ds.CreateScoreDB();

			// add new entry to table
			// ds.CreateScore(150); 

			// delete specific score
			// var deleteScore = ds.GetScores();
			// foreach( var s in deleteScore ){
			// 	if( s.Id == 8 ){
			// 		ds.DeleteScore( s ); 
			// 		break;
			// 	}
			// }

			// return all current entries in table
			ToConsole( "All Entries:");
			var scores = ds.GetScores(); 
			ToConsole( scores ); 

			// return all scores that match passed arg
			// ToConsole( "Get By Score" );
			// var myScore = ds.GetByScore(100); 
			// ToConsole( myScore ); 

			// return all scores that match date time arg
			// ToConsole( "Get By DateTime" );
			// Score dateScore = ds.GetSingleByDateTime(2016, 5, 1, 13, 36, 54); 
			// ToConsole( dateScore.ToString() ); 
		}
		

	// -----------------------------------------------------------------------------
	// Debug Methods

		// [[ ----- TO CONSOLE (table elements) ----- ]]
		private void ToConsole(IEnumerable<Score> _scores){
			List<Score> orderedScores = new List<Score>();

			// Arrange scores in ID order
			foreach( var score in _scores ){
				if( orderedScores.Count > 0 
					&& score.Id <= orderedScores[0].Id 
					){
					
					orderedScores.Insert(0, score);
				} else{
					orderedScores.Add(score); 
				}
			}

			foreach (var score in orderedScores) {
				ToConsole(score.ToString());
			}
		}

		// [[ ----- TO CONSOLE (string) ----- ]]
		private void ToConsole(string msg){
			DebugText.text += System.Environment.NewLine + msg;
			Debug.Log (msg);
		}
	}
}