﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

public class WaveController : MonoBehaviour {

	// Variables
	public static List<WaveNode> nodeList = new List<WaveNode>(); 
	public List<WaveSpike> spikeQueue = new List<WaveSpike>(); 
	public float lineStart, lineEnd; 
	public float nodeGapDist; 
	public int spikeLength;
	public float spikeSpeedInterval = 1; 
	public float currentSpikeInterval;
	public int daimondCounter = 0;
	public bool isBlack;

	// Reference Variables
	public GameObject nodePrefab; 


	// [[ ----- START ----- ]]
	void Awake () {

		// set spike interval
		currentSpikeInterval = spikeSpeedInterval; 

		nodeList = new List<WaveNode>();


	 	// generation code
	 	// if( !(nodeList.Count > 0) ){
		 	float xPos = lineStart; 
		 	while( xPos < lineEnd ){

		 		// instantiate node
		 		GameObject newNode = (GameObject) Instantiate(
		 			nodePrefab,
		 			new Vector2( xPos, 0 ),
		 			Quaternion.identity,
		 			transform
		 			);

		 		xPos += nodeGapDist; 

		 		// turn on daimond graphic
		 		daimondCounter++;
		 		if( daimondCounter > 4 ){
		 			daimondCounter = 0;
		 			string dColor = isBlack ? "BlackDaimond" : "WhiteDaimond";
		 			newNode.transform.Find(dColor).gameObject.SetActive(true);
		 			isBlack = !isBlack; 
		 		}
		 	}
		// }
	}
	
	// Update is called once per frame
	void Update () {

		// entropy
		// List<WaveSpike> deadSpikes = new List<WaveSpike>(); 
		// foreach( WaveSpike s in spikeQueue ){
		// 	if( s.DeathCheck(Time.deltaTime) ){
		// 		KillSpike(s); 
		// 		// Debug.Log("DEAD SPIKE" + null);
		// 		deadSpikes.Add(s);
		// 	}
		// }

		// foreach( WaveSpike s in deadSpikes ){
		// 	spikeQueue.Remove(s);
		// }

		/*currentSpikeInterval -= Time.deltaTime; 
		
		// move wave one forward
		if( currentSpikeInterval <= 0 ){
			currentSpikeInterval = spikeSpeedInterval; 

			// increment spike form, if any present
			if( spikeQueue.Count > 0 ){
				for( int i = 0; i < spikeQueue.Count; i++ ){
					// increase the key index for the spike by 1
					spikeQueue[i].nodePosIndex++;

					// for each node within the spike span
					for( int k = 0; k < spikeLength; k++ ){
						if( k + spikeQueue[i].nodePosIndex < nodeList.Count){
							nodeList[k + spikeQueue[i].nodePosIndex].SetTarget(
								spikeQueue[i].waveForm[k].y );
						
							// print( "Node: " + k + spikeQueue[i].nodePosIndex 
							// 	+ " y= " + spikeQueue[i].waveForm[k].y);
						}
					}
				}

				// remove any spikes that are finished
				if( spikeQueue.Count > 0 
					&&spikeQueue[0].nodePosIndex > nodeList.Count 
					){
					spikeQueue.RemoveAt(0);
				}
			}
		}*/
	}

	// [[ ----- SET FORM ----- ]]
	public void SetForm( float _frequency, float _magnitude, int _centerIndex ){
		// foreach( WaveNode node in nodeList ){
		// 	node.SetTarget( _frequency, _magnitude ); 
		// }

		// make spike
		print("making spike"); 
		float spikeXpos = 0;
		float spikeYpos = 0; 
		float lifespan = Mathf.Clamp(_magnitude, 3, 6); 
		WaveSpike newSpike = new WaveSpike( spikeLength, _centerIndex, lifespan ); 
		for( int i = 0; i < spikeLength; i++ ){
			spikeXpos = i * nodeGapDist; 
			spikeYpos = Mathf.Sin( spikeXpos * _frequency ) * _magnitude; 
			newSpike.AddPointToWave( new Vector2( spikeXpos, spikeYpos ) );
		}



		// add to list
		// spikeQueue.Add(newSpike);

		// animate spike
		int beginIndex = _centerIndex - (spikeLength/2);
		for( int i = 0; i < spikeLength; i++ ){
			if( beginIndex + i > 0 && beginIndex + i < WaveController.nodeList.Count  ){
				nodeList[i + beginIndex].SetTarget( newSpike.waveForm[i].y ); 
			}
		}

	}

	// [[ ----- SET SPIKE POSITION ----- ]]
	// private void SetSpikePosition(){
	// 	for( int i = 0; i < spikeLength; i++ ){
	// 		nodeList[i].SetTarget( newSpike.waveForm[i].y);
	// 		print("Node: " + i + " Set to y: " + newSpike.waveForm[i].y); 
	// 	}
	// }

	// [[ ----- KILL SPIKE ----- ]]
	private void KillSpike( WaveSpike _spike ){
		int beginIndex = _spike.nodePosIndex - (spikeLength/2); 
		for( int i = 0; i < spikeLength; i++ ){
			int nIndex = i + beginIndex; 
			if( nIndex > 0 && nIndex < WaveController.nodeList.Count ){
				nodeList[i + beginIndex].ResetPosition();
			}
		}
	}

	// [[ ----- RESET NODES ----- ]]
	public void ResetNodes(){
		
		spikeQueue.Clear(); 
		foreach( WaveNode node in nodeList ){
			node.ResetPosition();
		}
	}

	
}

public class WaveSpike {

	// Variables
	public Vector2[] waveForm;
	public int nodePosIndex = 0; 
	private int index;

	private float lifeRemaining;

	// [[ ----- CONSTRUCTOR ----- ]]
	public WaveSpike( int _length, int _centerIndex, float _lifespan ){
		waveForm = new Vector2[_length];
		index = 0; 
		nodePosIndex = _centerIndex; 
		lifeRemaining = _lifespan; 
	}

	// [[ ----- ADD POINT TO WAVE ----- ]]
	public void AddPointToWave( Vector2 _newPoint ){
		if( index < waveForm.Length ){
			waveForm[index] = _newPoint;
			index++; 
		} else { Debug.LogWarning("Out of Range!" + index );}
	}

	// [[ ----- DEATH CHECK ----- ]]
	public bool DeathCheck( float _deltaTime ){
		lifeRemaining -= _deltaTime; 
		if( lifeRemaining < 0 ){ return true; }
		else{ return false; }
	}
}
